#!/usr/bin/env python
import re
from pathlib import Path

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    with Path(__file__).parent.joinpath(*names).open(encoding=kwargs.get("encoding", "utf8")) as fh:
        return fh.read()


setup(
    name="archive-steam-reviews",
    version="0.2.1",
    license="MIT",
    description="Scrape all Steam reviews from a specific profile",
    long_description="{}\n{}".format(
        re.compile("^.. start-badges.*^.. end-badges", re.M | re.S).sub("", read("README.rst")),
        re.sub(":[a-z]+:`~?(.*?)`", r"``\1``", read("CHANGELOG.rst")),
    ),
    author="Manuel Grabowski",
    author_email="git@manuelgrabowski.de",
    maintainer="Brie Carranza",
    maintainer_email="hi@brie.ninja",
    url="https://github.com/manuelgrabowski/archive-steam-reviews",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[path.stem for path in Path("src").glob("*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: MacOS",
        "Operating System :: MacOS :: MacOS X",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Games/Entertainment",
        # uncomment if you test on these interpreters:
        # "Programming Language :: Python :: Implementation :: IronPython",
        # "Programming Language :: Python :: Implementation :: Jython",
        # "Programming Language :: Python :: Implementation :: Stackless",
        "Topic :: Utilities",
    ],
    project_urls={
        "Source Code": "https://github.com/manuelgrabowski/archive-steam-reviews",
        "Initial Release Post": "https://log.manuelgrabowski.de/post/archive-steam-reviews/",
        "Issue Tracker": "https://github.com/manuelgrabowski/archive-steam-reviews/issues",
    },
    keywords=[
        "steam","review","video game",
    ],
    python_requires=">=3.8",
    install_requires=[
        # eg: "aspectlib==1.1.1", "six>=1.7",
        "beautifulsoup4==4.12.2",
        "certifi==2023.7.22",
        "charset-normalizer==3.3.2",
        "idna==3.4",
        "markdownify==0.11.6",
        "python-dateutil==2.8.2",
        "requests==2.31.0",
        "six==1.16.0",
        "soupsieve==2.5",
        "urllib3==2.0.7",
    ],
    extras_require={
        # eg:
        #   "rst": ["docutils>=0.11"],
        #   ":python_version=="2.6"": ["argparse"],
    },
    entry_points={
        "console_scripts": [
            "archive-steam-reviews = archive_steam_reviews.cli:main",
        ]
    },
)



"""Scrapes Steam reviews for a user."""

import argparse
import re
import urllib.request
import os.path
import time
import json
import requests
from bs4 import BeautifulSoup
from markdownify import markdownify as md
from dateutil import parser

CACHE_FILE = '/tmp/appnames.json'

def is_older_than(file, days=1):
    """Checks if a file was modified within the last x days."""
    file_time = os.path.getmtime(file)
    return (time.time() - file_time) / 3600 > 24 * days


def cache_app_names():
    """Stores the big appid<->appname JSON from Steam locally."""
    source_url = 'https://api.steampowered.com/ISteamApps/GetAppList/v0002/?format=json'

    if not os.path.isfile(CACHE_FILE) or is_older_than(CACHE_FILE, 7):
        print("Refreshing app name cache (" + CACHE_FILE + ").")
        urllib.request.urlretrieve(source_url, CACHE_FILE)


def fallback_name_by_id_lookup(app_id):
    """Scrapes a game name from the corresponding store page."""
    name = 'unknown'

    url = 'https://store.steampowered.com/app/' + str(app_id)
    response = requests.get(url, timeout=10)
    soup = BeautifulSoup(response.content, "html.parser")
    name_div = soup.find("div", id="appHubAppName_responsive")

    if name_div:
        name = name_div.decode_contents()

    return name


def find_name_by_id(app_id):
    """Returns the game name for a given app_id, either from local cache or new scrape."""
    name = 'unknown'
    with open(CACHE_FILE, encoding="utf-8") as f:
        apps = json.load(f)['applist']['apps']
        for app in apps:
            if app['appid'] == app_id:
                name = app['name']
                break

    if name == 'unknown':
        name = fallback_name_by_id_lookup(app_id)

    return name


def parse_review_dates(review):
    """Extracts the date a review was published and last updated."""
    review_date_text = review.find("div", class_="posted").text.strip()
    review_date_regex = r"Posted (?P<review_date>.*?)\.(\s*Last edited (?P<last_updated>.*?)\.)?"
    matches = re.finditer(review_date_regex, review_date_text)
    review_date = None
    last_updated = None
    for m in matches:
        review_date = m.group('review_date')
        last_updated = m.group('last_updated')

    return review_date, last_updated


def parse_review_playtime(review):
    """Extracts the current total and "total at time of review" playtime."""
    playtime_text = review.find("div", class_="hours").text.strip()
    playtime_regex = r"(?P<total_playtime>.*?) hrs on record(\s*\((?P<playtime_at_review>.*?) hrs at review time\))?"
    matches = re.finditer(playtime_regex, playtime_text)
    total_playtime = None
    playtime_at_review = None
    for m in matches:
        total_playtime = m.group('total_playtime')
        playtime_at_review = m.group('playtime_at_review')
    return total_playtime, playtime_at_review


def parse_review(review, username):
    """Extracts all relevant raw data for a review."""
    steam_link = review.find("a", class_="game_capsule_ctn")["href"]
    steam_id = int(steam_link.split('/')[-1])

    review_text = md(
        review.find(
            "div",
            class_="content").decode_contents())

    review_date, last_updated = parse_review_dates(review)
    total_playtime, playtime_at_review = parse_review_playtime(review)

    return {
        "steam_link": "https://store.steampowered.com/app/" + str(steam_id),
        "review_link": "https://steamcommunity.com/id/" +
        username + "/recommended/" + str(steam_id) + "/",
        "app_name": find_name_by_id(steam_id),
        "total_playtime": total_playtime,
        "playtime_at_review": playtime_at_review,
        "review_text": review_text,
        "review_date": parser.parse(review_date),
        "last_updated": None if not last_updated or (
            last_updated == review_date) else parser.parse(last_updated)}


def scrape_steam_reviews(username, download_all):
    """Scrapes Steam reviews for a given username."""
    base_url = f"https://steamcommunity.com/id/{username}/recommended/?p="
    page_number = 1
    parsed_reviews = []

    while True:
        url = base_url + str(page_number)
        response = requests.get(url, timeout=10)
        soup = BeautifulSoup(response.content, "html.parser")
        reviews = soup.find_all("div", class_="review_box")

        # TODO: Proper check based on " Showing 1-10 of 55 entries " text
        if not reviews:
            break

        for review in reviews:
            parsed_reviews.append(parse_review(review, username))

        if download_all:
            page_number += 1
        else:
            break

    return parsed_reviews


def print_review(review):
    """Prints a parsed review to stdout."""
    print(f"Game: {review['app_name']}")
    print(f"Steam Link: {review['steam_link']} / {review['review_link']}")
    print(
        f"Review Date: {review['review_date']} (last updated: {review['last_updated']})")
    print(
        f"Playtime: {review['total_playtime']} (at review: {review['playtime_at_review']})")
    print(f"Review: {review['review_text']}")
    print("-" * 50)


def save_review(review):
    """Saves a parsed review into a Hugo-compatible Markdown file."""
    steam_id = review['steam_link'].split('/')[-1]

    with open(steam_id + '.md', mode='w', encoding="utf-8") as f:
        f.write('---')
        f.write('\n')
        f.write('title: "' + review['app_name'] + '"')
        f.write('\n')
        f.write('steam_link: ' + review['steam_link'])
        f.write('\n')
        f.write('review_link: ' + review['review_link'])
        f.write('\n')
        f.write('date: ' + review['review_date'].strftime("%Y-%m-%d"))
        if review['last_updated']:
            f.write('\n')
            f.write(
                'last_updated: ' +
                review['last_updated'].strftime("%Y-%m-%d"))
        f.write('\n')
        f.write('total_playtime: ' + review['total_playtime'])
        if review['playtime_at_review']:
            f.write('\n')
            f.write('playtime_at_review: ' + review['playtime_at_review'])
        f.write('\n')
        f.write('---')
        f.write('\n')
        f.write(review['review_text'])
        f.write('\n')


def main():
    """The main method."""
    arg_parser = argparse.ArgumentParser(
        description="Archive Steam reviews from a specific user account.")
    arg_parser.add_argument(
        "--username",
        help="Steam username for which to download reviews")
    arg_parser.add_argument(
        '--all',
        default=False,
        action='store_true',
        help="download all reviews (or just the first page)")
    arg_parser.add_argument(
        '--save',
        default=False,
        action='store_true',
        help="save downloaded reviews to filesystem (or print to stdout)")
    args = arg_parser.parse_args()

    cache_app_names()

    reviews = scrape_steam_reviews(args.username, args.all)

    for review in reviews:
        if args.save:
            save_review(review)
        else:
            print_review(review)